<?
/******************************************************************************\
 *                         SQL Database Thread Class                          *
 ******************************************************************************
 * This class is abstract and must be extended by exact DB classes.           *
 ******************************************************************************
 * Written by Vladimir Sibirov a.k.a. Trustmaster.                            *
\******************************************************************************/

class SQLDB
{
        //PHP5: private

        // DB link (resource) identifier
        var $base = null;
        // DB result identifier
        var $result = null;
        // Number of queries used
        var $query_num = 0;
        // Hostname for connection
        var $host = 'localhost';
        // Username for login
        var $user = '';
        // Password for login
        var $pass = '';
        // Database name
        var $name = '';

        // Constructor
        //PHP5: public function  __construct()
        function SQLDB($host = null, $user = null, $pass = null, $name = null)
        {
                // If constructor is called without params we try to load them
                // from config constants.
                if(is_null($host) && defined('SQLDB_HOST')) $this->host = SQLDB_HOST;
                elseif(!is_null($host)) $this->host = $host;

                if(is_null($user) && defined('SQLDB_USER')) $this->user = SQLDB_USER;
                elseif(!is_null($user)) $this->user = $user;

                if(is_null($pass) && defined('SQLDB_PASS')) $this->pass = SQLDB_PASS;
                elseif(!is_null($pass)) $this->pass = $pass;

                if(is_null($name) && defined('SQLDB_NAME')) $this->name = SQLDB_NAME;
                elseif(!is_null($name)) $this->name = $name;

                // Then DB-specific code goes
        }

        // Runs SQL query
        // PHP5: public
        function Query($query)
        {
                $this->query_num++;
                // DB-specific code will go here
        }

        // Converts PHP vars to SQL INSERT query and runs it
        function InsertQuery($table, $values)
        {
                $sql = 'INSERT INTO '.$this->EscapeString($table).' ';
                $keys = '';
                $vals = '';
                foreach($values as $key => $val)
                {
                        $keys .= $this->EscapeString($key).', ';
                        if(is_string($val)) $vals .= "'".$this->EscapeString($val)."', ";
                        else $vals .= $this->EscapeString($val).', ';
                }
                // Trimming
                $keys = substr($keys, 0, strlen($keys) - 2);
                $vals = substr($vals, 0, strlen($vals) - 2);
                $sql .= '('.$keys.') VALUES ('.$vals.')';
                $this->Query($sql);
        }

        // Converts PHP vars to SQL UPDATE query and runs it
        function UpdateQuery($table, $values, $where = null)
        {
                $sql = 'UPDATE '.$this->EscapeString($table).' SET ';
                foreach($values as $key => $val)
                {
                        $sql .= $this->EscapeString($key).' = ';
                        if(is_string($val)) $sql .= "'".$this->EscapeString($val)."', ";
                        else $sql .= $this->EscapeString($val).', ';
                }
                $sql = substr($sql, 0, strlen($sql) - 2);
                if(!is_null($where))
                {
                        $sql .= ' WHERE ';
                        foreach($where as $key => $val)
                        {
                                $sql .= $this->EscapeString($key).' = ';
                                if(is_string($val)) $sql .= "'".$this->EscapeString($val)."' AND ";
                                else $sql .= $this->EscapeString($val).', ';
                        }
                        $sql = substr($sql, 0, strlen($sql) - 5);
                }
                $this->Query($sql);
        }

        // Escapes special characters for use in SQL statement
        // PHP5: public
        function EscapeString($str)
        {
                // Don't use regexp! They are much slower than str functions
                $result = str_replace("'", "\'", $str);
                $result = str_replace("`", "\`", $result);
                return $result;
        }

        // Terminates program and outputs error message
        // PHP5: public
        function FatalError()
        {
                die('SQLDB Error #'.$this->Errno().': '.$this->Error());
        }

        // Returns total number of queries since initialization
        // PHP5: public
        function NumQueries()
        {
                return $this->query_num;
        }

        // Closes connection
        // PHP5: public
        function Close()
        {
                $this->Free();
                // Here we call DB-specific functions if needed
        }

        // Returns last error string
        // PHP5: abstract
        function Error()
        {
                // This method is abstract
        }

        // Returns last error number
        // PHP5: abstract
        function Errno()
        {
                // This method is abstract
        }

        // Returns number of affected rows
        // PHP5: abstract
        function Affected()
        {
                // This method is abstract
        }

        // Returns number of rows in result set
        // PHP5: abstract
        function NumRows()
        {
                // This method is abstract
        }

        // Returns single field from result set
        // PHP5: public
        function Field($query = '')
        {
                if(!empty($query)) $this->Query($query);
                if(!$this->result) $this->FatalError();
                // Then DB-specific code goes
        }

        // Returns single row from result set
        // PHP5: public
        function Row($query = '')
        {
                if(!empty($query)) $this->Query($query);
                if(!$this->result) $this->FatalError();
                // Then DB-specific code goes
        }

        // Return result set as array of rows
        // PHP5: public
        function RowTable($query = '')
        {
                if(!empty($query)) $this->Query($query);
                if(!$this->result) $this->FatalError();
                // Then DB-specific code goes
        }

        // Returns accociative array from result set
        // PHP5: public
        function AssocArray($query = '')
        {
                if(!empty($query)) $this->Query($query);
                if(!$this->result) $this->FatalError();
                // Then DB-specific code goes
        }

        // Returns result set as array of associative arrays
        // PHP5: public
        function ArrayTable($query = '')
        {
                if(!empty($query)) $this->Query($query);
                if(!$this->result) $this->FatalError();
                // Then DB-specific code goes
        }

        // Frees result set
        // PHP5: abstract
        function Free()
        {
                // This method is abstract
        }
}
?>